/****************************************************************************
 *
 *   Copyright (C) 2003, 2004 Wolfgang G. Reissnegger, All Rights Reserved
 *   LCD key support added by Mike Willett <mike@digital-chaos.co.uk>
 *
 *   gnagflow@users.sourceforge.net                                    
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

// Compile only, if this driver is enabled
//
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef AUDIOBOX_ENABLE_CONSOLE

#include <unistd.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "socket.h"
#include "keytab.h"
#include "display_console.h"

// These includes provide the context object definitions
//
#include "song.h"
#include "info.h"
#include "search.h"
#include "menu.h"
#include "main.h"

#include "util.h"


/******************************************************************************/

static unsigned int console_scroll_next;	// time for the next scroll event
static int console_scroll_delta	= 1;		// 1 or -1, will be added to idx
static int console_scroll_idx	= 0;		// scroll index

/******************************************************************************/
const char *const cuon = "\033[?;25h";
const char *const green = "\033[1;40;32m";
const char *const invgrn = "\033[1;42;30m";
const char *const normal = "\033[0m";
const char *const cuoff = "\033[?;25l";

#define X0 10
#define Y0 15
#define WIDTH 40

// these macros need some reworking... we really don't need both of them.
//
#define xy_printf(_col,_line,_arg...)           \
    {   \
        char _buf[MAX_PATH_LEN];    \
        sprintf(_buf,_arg);         \
        printf("\033[%d;%dH\033[0;42;30m%-40.40s\033[0m",_line+1+18,_col+1+20,_buf); \
    }
//	printf("%3i,%3i,%s\n",_line,_col,_buf);

#define console_printf(_arg...)           \
    {   \
        printf(_arg); \
    }

static int console_display_open(display_t * display);
static void console_display_close(display_t * display);
static int console_display_fd(display_t * display);
static void console_display_handle_event(display_t * display);
static void console_display_show(display_t * display, void *ctx, unsigned int type);
static void console_display_cmd(display_t * display, unsigned int cmd, void *arg);


/******************************************************************************/

// This structure defines the generic LCDproc display attributes.
//
struct console_display_d
{
	display_t parent;

	int sock;		// socket fd
	int lines;		// number of display lines
	int columns;		// number of display columns

#define m_sock      ((struct console_display_d *) display)->sock
#define m_lines     ((struct console_display_d *) display)->lines
#define m_columns   ((struct console_display_d *) display)->columns
};


struct console_display_d console_display = {
	// intialization of parent 'class'
	//
	{
	      open:		console_display_open,
	      close:		console_display_close,
	      fd:		console_display_fd,
	      ev_handler:	console_display_handle_event,
	      show:		console_display_show,
	      cmd:		console_display_cmd,
	 },

	-1,			// sock
	2,			// lines
	40			// columns
};


/******************************************************************************/

static int console_display_open(display_t * display)
{
	// set default LCD size
	m_columns = 40;
	m_lines = 6;

	return 0;
}


/******************************************************************************/

static void console_display_close(display_t * display)
{
}


/******************************************************************************/

static int console_display_fd(display_t * display)
{
	return 0; //m_sock;
}


/******************************************************************************/

static unsigned int console_display_translate_key(const char key)
{
	return 0;
}


/******************************************************************************/

// The token handling code below has been snipped out of the LCDproc client.
//

static void console_display_handle_event(display_t * display)
{
}


/******************************************************************************
 *
 * SHOW functions for the LCD driver
 *
 ******************************************************************************/

static void console_display_show_menu(display_t * display, void *ctx)
{
	int i;
	int line;

	// If we have a 1 line display, we do not bother about what to display,
	// we simply display the current entry...
	//
	if (1 == m_lines)
	{
		file_entry_t *entry = list_data(list_nth(c_start, c_selection));

		if (entry)
		{
			char *name;

			name = c_view_fullpath ? entry->fullpath : entry->name;

			xy_printf(1, 0, ">%s%s",
				  FILE_TYPE_DIR == file_type(entry) ? "+" : "",
				  &name[c_left_margin]);
		}
	}

	// Check, if the selection is still within our display area
	//
	if (c_display_top_line + m_lines <= c_selection)
	{
		// adjust downwards
		//
		c_display_top_line = c_selection - (m_lines - 1);
	}
	else if (c_display_top_line > c_selection)
	{
		// adjust upwards
		//
		c_display_top_line = c_selection;
	}


	// Loop through the visible lines and print them
	//
	for (i = 0, line = c_display_top_line; i < m_lines; i++, line++)
	{
		file_entry_t *entry = list_data(list_nth(c_start, line));

		if (entry)
		{
			char *name;

			name = c_view_fullpath ? entry->fullpath : entry->name;

			xy_printf(1, i, "%c%s%s",
				  c_selection == i + c_display_top_line ?
				  entry->marked ? '#' : '>' :
				  entry->marked ? '*' : ' ',
				  FILE_TYPE_DIR == file_type(entry) ? "+" : "",
				  &name[c_left_margin]);
		}
		else
		{
			xy_printf(1, i, "-");
		}
	}
}


/******************************************************************************/

static void console_display_show_search(display_t * display, void *ctx)
{
	// For 1 line displays, we'll have to save some space
	//
	if (1 == m_lines)
	{
		xy_printf(1, 0, "S(%c):>%s<", SEARCH_FULL == c_search_mode ? 'f' : 's',
			  c_search_buf);
	}
	else
	{
		xy_printf(1, 0, "Search for (%s):", SEARCH_FULL == c_search_mode ? "full" : "sub");
		xy_printf(1, 1, ">%s<", c_search_buf);
	}
}


/******************************************************************************/

static void console_display_show_info(display_t * display, void *ctx)
{
	int line;

	for (line = 0; line < m_lines; ++line)
	{
		xy_printf(1, line, c_text[line]);
	}
}


/******************************************************************************/

static void console_display_show_song(display_t * display, void *ctx)
{
	unsigned int ms_now;
	const char *pause_str = "";
	unsigned int total_name_row_len;

	// We wait a couple of seconds before we start scrolling
	// the title
	//
	ms_now = get_timestamp();


	// Initialize total row len with name length
	//
	total_name_row_len = c_name_len;

	// Set up the PAUSE string, if needed
	//
	if (SONG_MODE_PAUSE == c_current_mode)
	{
		pause_str = "PAUSED ";

		// Add length of pause string
		//
		total_name_row_len += 7;
	}


	// If the song name fits into the line, we do not scroll.
	//
	if (total_name_row_len <= m_columns)
	{
		// Display the song name 
		//
		xy_printf(1, 0, "%s%s", pause_str, c_name);
	}
	else
	{
		// Check, if the next scroll event is already due.
		//
		if (ms_now > console_scroll_next)
		{
			// calculate the new index and the scroll direction
			//
			console_scroll_idx += console_scroll_delta;

			if (0 == console_scroll_idx)
			{
				// We hit the beginning of the song name
				//
				console_scroll_delta = 1;

				// In the case we hit the beginning of the song
				// name, we wait an additional time before we
				// start scrolling again.
				//
				console_scroll_next = ms_now + SONG_SCROLL_EDGE_WAIT;
			}
			else if (total_name_row_len - m_columns == console_scroll_idx)
			{
				// Reached the end of the song name
				//
				console_scroll_delta = -1;

				// In the case we hit the end of the song name,
				// we wait an additional time before we start
				// scrolling again.
				//
				console_scroll_next = ms_now + SONG_SCROLL_EDGE_WAIT;
			}
			else
			{
				// Set current timestamp
				//
				console_scroll_next = ms_now + SONG_SCROLL_DELAY;
			}
		}
		xy_printf(1, 0, "%s%s", pause_str, &c_name[console_scroll_idx]);
	}

	if (c_info_show)
	{
		// Show the mp3 info
		//
		xy_printf(1, 1, "%dkBit %d.%01dkHz",
			  c_bit_rate, c_sample_rate / 1000, (c_sample_rate % 1000) / 100);
	}
	else
	{
		// Show the time and position
		//
		xy_printf(1, 1, "%s %s%s%d/%d",
			  c_time_show_remaining ? c_time_remaining : c_time,
			  c_play_random ? "R" : "", 
			  c_play_loop ? "L" : "", 
			  c_file_idx_selected + 1, 
			  list_len(c_file_list));
	}
}


/******************************************************************************/

static void console_display_show(display_t * display, void *ctx, unsigned int type)
{
	assert(NULL != ctx);
	assert(NULL != display);

	switch (type)
	{
	case TYPE_MENU:
	case TYPE_SONGLIST_MENU:
		console_display_show_menu(display, ctx);
		break;
	case TYPE_SEARCH:
		console_display_show_search(display, ctx);
		break;
	case TYPE_INFO:
		console_display_show_info(display, ctx);
		break;
	case TYPE_SONG:
		console_display_show_song(display, ctx);
		break;

	default:
		// ignore
		;
	}
}


/******************************************************************************/

static void console_display_cmd(display_t * display, unsigned int cmd, void *arg)
{
	console_scroll_idx	= 0;
	console_scroll_delta	= 1;
	
	// Set the initial time it takes before the song name starts scrolling.
	//
	console_scroll_next	= get_timestamp() + SONG_SCROLL_INITIAL_WAIT;
}


#endif // #ifdef AUDIOBOX_ENABLE_CONSOLE
