/****************************************************************************
 *
 *   Copyright (C) 2003, 2004 Wolfgang G. Reissnegger, All Rights Reserved
 *
 *   gnagflow@users.sourceforge.net                                    
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#include "session_menu.h"

#include <stdio.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "display.h"
#include "menu.h"
#include "info.h"
#include "hotkey.h"
#include "util.h"


/******************************************************************************/

static void session_switch_to(unsigned int idx)
{
	char text[32];

	display_session_set(idx);

	// Display the session number. The use gets to see the 'real' number
	// (e.g. idx +1) so he won't be confused by SESSION #0...
	//
	sprintf(text, "SESSION #%d", idx + 1);
	display_push(info_new(text, 500));
}

static void session_1(void *data)
{
	session_switch_to(0);
}
static void session_2(void *data)
{
	session_switch_to(1);
}
static void session_3(void *data)
{
	session_switch_to(2);
}
static void session_4(void *data)
{
	session_switch_to(3);
}
static void session_5(void *data)
{
	session_switch_to(4);
}
static void session_6(void *data)
{
	session_switch_to(5);
}
static void session_7(void *data)
{
	session_switch_to(6);
}
static void session_8(void *data)
{
	session_switch_to(7);
}

/******************************************************************************/

static void session_next(void *data)
{
	session_switch_to((display_session_idx() + 1) % NUM_SESSION);
}


/******************************************************************************/

static void session_prev(void *data)
{
	session_switch_to((display_session_idx() - 1) % NUM_SESSION);
}


/******************************************************************************/

void session_menu(void *data)
{
	screen_t *session_menu = menu_new();

	menu_add(session_menu, "Next     SESSION", session_next);
	menu_add(session_menu, "Previous SESSION", session_prev);
	menu_add(session_menu, "Switch to SESSION 1", session_1);
	menu_add(session_menu, "Switch to SESSION 2", session_2);
	menu_add(session_menu, "Switch to SESSION 3", session_3);
	menu_add(session_menu, "Switch to SESSION 4", session_4);
	menu_add(session_menu, "Switch to SESSION 5", session_5);
	menu_add(session_menu, "Switch to SESSION 6", session_6);
	menu_add(session_menu, "Switch to SESSION 7", session_7);
	menu_add(session_menu, "Switch to SESSION 8", session_8);

	display_push(session_menu);
}

/******************************************************************************/

void session_menu_init()
{
	hotkey_add(KEY_SESSION_MENU, session_menu);
	hotkey_add(KEY_SESSION_MENU_NEXT, session_next);
	hotkey_add(KEY_SESSION_MENU_PREV, session_prev);
}
