/****************************************************************************
 *
 *   Copyright (C) 2003, 2004 Wolfgang G. Reissnegger, All Rights Reserved
 *
 *   gnagflow@users.sourceforge.net                                    
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#ifndef SONG_H_
#define SONG_H_

#include "list.h"
#include "fileutil.h"
#include "display.h"


// Song context object, derived from display context
//
typedef struct song_display_context_d
{
	// Song name currently DISPLAYED. This is not necessarily the
	// song PLAYED BACK...
	//
	char name[MAX_PATH_LEN];
	int name_len;
	int name_show_fullpath;

	list_t *file_list;
	int file_idx_playing;	// song currently playing
	int file_idx_selected;	// song currently displayed

	int current_mode;	// PLAY, STOP, PAUSE
#define SONG_MODE_PLAY      1
#define SONG_MODE_STOP      2
#define SONG_MODE_PAUSE     3
	int play_random;	// random play flag
	int play_loop;		// play songs in a loop

	int time_show_remaining;	// time display flag
	int info_show;		// show info flag

	int sample_rate;	// sample rate (e.g. 44100)
	int bit_rate;		// mp3 bit rate (e.g. 128)

	char time[32];		// time
	char time_remaining[32];	// remaining play time


// These are accessor macros. Make sure that ctx is of type
// song_display_context_t* when using these...
//
#define c_name                  ((song_display_context_t *) ctx)->name
#define c_name_len              ((song_display_context_t *) ctx)->name_len
#define c_name_show_fullpath    ((song_display_context_t *) ctx)->name_show_fullpath
#define c_file_list             ((song_display_context_t *) ctx)->file_list
#define c_file_idx_playing      ((song_display_context_t *) ctx)->file_idx_playing
#define c_file_idx_selected     ((song_display_context_t *) ctx)->file_idx_selected
#define c_current_mode          ((song_display_context_t *) ctx)->current_mode
#define c_play_random           ((song_display_context_t *) ctx)->play_random
#define c_play_loop             ((song_display_context_t *) ctx)->play_loop
#define c_time_show_remaining   ((song_display_context_t *) ctx)->time_show_remaining
#define c_info_show             ((song_display_context_t *) ctx)->info_show
#define c_sample_rate           ((song_display_context_t *) ctx)->sample_rate
#define c_bit_rate              ((song_display_context_t *) ctx)->bit_rate
#define c_time                  ((song_display_context_t *) ctx)->time
#define c_time_remaining        ((song_display_context_t *) ctx)->time_remaining
}
song_display_context_t;


/******************************************************************************/

#define TYPE_SONG               0xda91876b


/******************************************************************************/

screen_t *song_new();
screen_t *song_active_screen();

void song_file_list_add(screen_t * screen, file_entry_t * entry);
void song_set_file(screen_t * screen, file_entry_t * entry);
void song_set_file_list(screen_t * screen, list_t *list, int current);
void song_parse(screen_t * screen, const char *buf);
void song_play(screen_t * screen);
void song_suspend(screen_t * screen);
void song_resume(screen_t * screen);

#endif
